from typing import TYPE_CHECKING, ClassVar

from pydantic import BaseModel

if TYPE_CHECKING:
    from tygle.base.rest import REST


class Resource(BaseModel):
    rest: ClassVar["REST"]

    class Config:
        use_enum_values = True



from aiogoogle import GoogleAPI
from tygle.client import Client


class REST:
    def __init__(self, client: Client, parent: GoogleAPI) -> None:
        self.client = client
        self.parent = parent



from typing import TYPE_CHECKING, Callable, Optional

from aiogoogle import Aiogoogle
from aiogoogle.auth.creds import ClientCreds, ServiceAccountCreds, UserCreds

if TYPE_CHECKING:
    from tygle.base import Request, Resource


class Client:
    """
    Proxy for interaction with Aiogoogle.
    """

    def __init__(
        self,
        *,
        api_key: Optional[str] = None,
        user_creds: Optional[UserCreds] = None,
        client_creds: Optional[ClientCreds] = None,
        service_account_creds: Optional[ServiceAccountCreds] = None,
    ) -> None:
        self.aiogoogle = Aiogoogle(
            api_key=api_key,
            user_creds=user_creds,
            client_creds=client_creds,
            service_account_creds=service_account_creds,
        )

    async def __aenter__(self, *args):
        await self.aiogoogle.__aenter__(*args)
        return self.aiogoogle

    async def __aexit__(self, *args):
        await self.aiogoogle.__aexit__(*args)

    async def __execute(self, method: Callable, request: "Request"):
        async with self as c:
            response = await method(c, request.request)
        return response

    async def as_api_key(self, request: "Request") -> tuple["Resource"]:
        return await self.__execute(Aiogoogle.as_api_key, request)

    async def as_user(self, request: "Request") -> tuple["Resource"]:
        return await self.__execute(Aiogoogle.as_user, request)

    async def as_service_account(self, request: "Request") -> tuple["Resource"]:
        return await self.__execute(Aiogoogle.as_service_account, request)

    async def as_anon(self, request: "Request") -> tuple["Resource"]:
        return await self.__execute(Aiogoogle.as_anon, request)



from abc import ABCMeta, abstractmethod
from pathlib import Path
from typing import Generic, Type, TypeVar

from aiofiles.tempfile import AsyncBufferedIOBase, AsyncBufferedReader
from aiogoogle.models import Request as AiogoogleRequest
from pydantic import BaseModel
from tygle.client import Client

RequestT = TypeVar("RequestT")


class Request(Generic[RequestT], metaclass=ABCMeta):
    def __init__(self, client: Client, request: AiogoogleRequest):
        self.client = client
        self.request = request

    @abstractmethod
    def post_process(self, response) -> RequestT:
        raise NotImplementedError

    async def as_api_key(self):
        response = await self.client.as_api_key(self)
        return self.post_process(response)

    async def as_user(self):
        response = await self.client.as_user(self)
        return self.post_process(response)

    async def as_service_account(self):
        response = await self.client.as_service_account(self)
        return self.post_process(response)

    async def as_anon(self):
        response = await self.client.as_anon(self)
        return self.post_process(response)


DataT = TypeVar("DataT", bound=BaseModel)


class DataRequest(Request[DataT]):
    def __init__(
        self,
        client: Client,
        request: AiogoogleRequest,
        resource_type: Type[DataT],
    ):
        self.resource_type = resource_type
        super().__init__(client, request)

    def post_process(self, response) -> DataT:
        return self.resource_type.parse_obj(response)


class PathRequest(Request[Path]):
    def post_process(self, response):
        path = response.media_download
        return Path(path)


class BufferRequest(Request[AsyncBufferedReader]):
    def post_process(self, response: AiogoogleRequest) -> AsyncBufferedReader:
        buffer: AsyncBufferedIOBase = response.media_download
        return AsyncBufferedReader(buffer._file, buffer._loop, buffer._executor)



from typing import Any, Generic, TypeVar

from .request import Request

RequestT = TypeVar("RequestT", bound=Request)


class Response(Generic[RequestT]):
    def __init__(self, request: RequestT, data: Any) -> None:
        self.request = request
        self.data = data



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tygle', 'tygle.base']

package_data = \
{'': ['*']}

install_requires = \
['aiogoogle==4.0.0', 'pydantic>=1.9.0,<2.0.0']

setup_kwargs = {
    'name': 'tygle',
    'version': '0.1.0',
    'description': 'Typed Google or simply tygle.',
    'long_description': '# **tygle**\n\n**Ty**ped Goo**gle** or simply *~~tiger~~* tygle.  \nCreated using [Aiogoogle](https://github.com/omarryhan/aiogoogle) for interaction with Google APIs and [pydantic](https://github.com/samuelcolvin/pydantic) for typing.\n\n## Work In Progress\n🛑 This is a WIP package. Expect bugs and changes.\n\n## Installation\nIn order to install the package, install tygle itself and APIs you need.  \nFor example:  \n`pip install tygle[tygle-sheets]`\n\n## Supported APIs\nCurrently, tygle only supports following APIs:  \n1. Sheets\n2. Drive',
    'author': 'shmookoff',
    'author_email': 'shmookoff@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/typed-google/tygle',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from aiogoogle import GoogleAPI
from tygle.client import Client


class API:
    api_name: str
    api_version: str

    def __init__(self, client: Client, api: GoogleAPI):
        self.client = client
        self.api = api

    @classmethod
    async def connect(cls, client: Client):
        async with client as c:
            return await c.discover(cls.api_name, cls.api_version)

    @classmethod
    async def new(cls, client: Client):
        api = await cls.connect(client)
        return cls(client, api)
